clc;clear; % clean the screen and workspace
load CalibTransducer.mat; % load the calibration parameters
close all; % close all the figures
delete(instrfindall); % delete all the instruments
fid = serial('com8'); % open serial port 
s.BytesAvailableFcnMode = 'byte'; % set the parameters of the serial port
s.InputBufferSize = 5000; 
s.BytesAvailableFcnCount = 10; 
s.BaudRate = 9600; 
s.BytesAvailableFcn = @my_callback; 
s.Timeout = 3000;
fopen(fid);
fprintf('Please type in the time length of this data collection\n'); % prompt the user to type in the data length 
timeout = input('Time length in minutes: '); 
timeout_used = timeout * 60 ; % convert minutes to seconds
fprintf(fid,'c\n');
check = [];
scaler = 5/255* 393.3217; %convert the units to mmHg
sample_rate = 44; % samples/second
while ((isempty(check)) || (check(1) ~= 'P')) % check the current state of the whole system 
    check = fscanf(fid, '%s');
    P_diff_location = strfind(check, 'P_diff'); % parse the string
    dP_MA_location = strfind(check, 'dP_MA');
    delta_location = strfind(check, 'delta');
    Gain_location = strfind(check, 'Gain');
    Bias_location = strfind(check, 'Bias');
    Thres_location = strfind(check, 'Thres');
    P_diff_current = str2double(check((P_diff_location + 6):(dP_MA_location - 1)));
    dP_MA_current = str2double(check((dP_MA_location + 5):(delta_location - 1)));
    delta_current = str2double(check((delta_location + 5):(Gain_location - 1)));
    Gain_current = str2double(check((Gain_location + 4):(Bias_location - 1)));
    Bias_current = str2double(check((Bias_location + 4):(Thres_location - 1)));
    Thres_current = str2double(check((Thres_location + 5):length(check))) * scaler - 388.2081;
end
fprintf('\nThe current values of P_diff, dP_MA, delta, Gain,and Bias are as follows:'); % display the current state
fprintf('\n\t\t\t\tP_diff = %f', P_diff_current);
fprintf('\n\t\t\t\tdP_MA = %f', dP_MA_current);
fprintf('\n\t\t\t\tdelta = %f', delta_current);
fprintf('\n\t\t\t\tGain = %f', Gain_current);
fprintf('\n\t\t\t\tBias = %f', Bias_current);
fprintf('\n\t\t\t\tThreshold Aorta Pressure = %f', Thres_current );

fprintf('\n*****************************************************************\n\r');
fprintf('\rDo you want to change the current values?\n') % ask the user to change value
check_flag = input('Please input 1 or 0 (1 for changing values)');
if(check_flag ~= 0)
    finish_input_flag = 0;
    while(finish_input_flag == 0)
        fprintf('\nPlease input the value of P_diff (the default value is 100)\n');
        P_diff = input('P_diff = ');
        if (size(P_diff) == 0)
            P_diff_used = 100;
        else
            P_diff_used = P_diff;        
        end
        
        fprintf('\nPlease input the value of delta (the default value is 1)\n');
        delta = input('delta = ');
        if (size(delta) == 0)
            delta_used = 1 ;
        else
            delta_used = delta;
        end
        
        fprintf('\nPlease input the value of the threshold aorta pressure (the default prssure = 77.5723989 mmHg)\n');
        thres = input('thres = ');
        if (size(thres) == 0)
            Thres_used = 77.5723989 ;
        else
            Thres_used = thres;
        end
        
        fprintf('P_diff = %f, delta = %f, threshold aorta voltage = %f\nIs this correct?\n', ...
            P_diff_used, delta_used, Thres_used );
        finish_input_flag = input('Please input 1 or 0 (1 for correct)\n');
        if ((finish_input_flag ~= 0))
            clear delta P_diff finish_input_flag thres;   
            break;    
        end
    end
else
    P_diff_used = P_diff_current;    
    delta_used = delta_current;
    Thres_used = Thres_current;
end
R_Use = 250;
Gain_total = a_Transducer * R_Calib / R_Use * 5 /255;
Bias_used = b_Transducer;
Thres_convert = round((Thres_used + 388.2081) / 393.3217 * 255 / 5);
send_packet = sprintf('P%06.2fC%06.3fD%dG%06.3fB%06.3fT%d', P_diff_used, dP_MA, delta_used, Gain_total, Bias_used, Thres_convert);
fprintf(fid,'o%s\r', send_packet); % send the new value to microcontroller
clear P_diff_current dP_MA_current delta_current %clear the internal variables
clear Gain_current Bias_current
clear P_diff_location dP_MA_location delta_location 
clear Gain_location Bias_location Thres_current

%initialize the vectors%initialize the vectors
Low_tank = zeros(timeout_used * sample_rate, 1);
High_tank = zeros(timeout_used * sample_rate, 1);
Ventricle = zeros(timeout_used * sample_rate, 1);
Aorta = zeros(timeout_used * sample_rate, 1);
PumpSignal = zeros(timeout_used * sample_rate, 1);
i_real = 1;

% set the figure handles
figureHandle1 = figure('NumberTitle','off',...
    'Name','High Tank Pressure',...
    'Color',[0 0 0],'Visible','on');
axesHandle1 = axes('Parent',figureHandle1,...
    'YGrid','on',...
    'YColor',[0.9725 0.9725 0.9725],...
    'XGrid','on',...
    'XColor',[0.9725 0.9725 0.9725],...
    'Color',[0 0 0],'xlim', [0, timeout_used], 'ylim', [0, 120] * scaler-388.2081);
hold on;
plotHandle1 = plot(axesHandle1,i_real,High_tank(1,1),'g',i_real,PumpSignal(1,1),'r','Marker','.','LineWidth',1);
xlabel('Time in seconds','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
ylabel('High Tank Pressure in mmHg','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
title('High Tank Pressure','FontSize',15,'Color',[1 1 0]);

figureHandle2 = figure('NumberTitle','off',...
    'Name','Low Tank Pressure',...
    'Color',[0 0 0],'Visible','on');
axesHandle2 = axes('Parent',figureHandle2,...
    'YGrid','on',...
    'YColor',[0.9725 0.9725 0.9725],...
    'XGrid','on',...
    'XColor',[0.9725 0.9725 0.9725],...
    'Color',[0 0 0],'xlim', [0, timeout_used], 'ylim', [0, 120] * scaler-388.2081);
hold on;
plotHandle2 = plot(axesHandle2,i_real,Low_tank(1,1),'g',i_real,PumpSignal(1,1),'r','Marker','.','LineWidth',1);
xlabel('Time in seconds','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
ylabel('Low Tank Pressure in mmHg','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
title('Low Tank Pressure','FontSize',15,'Color',[1 1 0]);


figureHandle3 = figure('NumberTitle','off',...
    'Name','Ventricle Pressure',...
    'Color',[0 0 0],'Visible','on');
axesHandle3 = axes('Parent',figureHandle3,...
    'YGrid','on',...
    'YColor',[0.9725 0.9725 0.9725],...
    'XGrid','on',...
    'XColor',[0.9725 0.9725 0.9725],...
    'Color',[0 0 0],'xlim', [0, timeout_used], 'ylim', [0, 120] * scaler-388.2081);
hold on;
plotHandle3 = plot(axesHandle3,i_real,Ventricle(1,1),'Marker','.','LineWidth',1,'Color',[0 1 0]);
xlabel('Time in seconds','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
ylabel('Ventricle Pressure in mmHg','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
title('Ventricle Pressure','FontSize',15,'Color',[1 1 0]);

figureHandle4 = figure('NumberTitle','off',...
    'Name','Aorta Pressure',...
    'Color',[0 0 0],'Visible','on');
axesHandle4 = axes('Parent',figureHandle4,...
    'YGrid','on',...
    'YColor',[0.9725 0.9725 0.9725],...
    'XGrid','on',...
    'XColor',[0.9725 0.9725 0.9725],...
    'Color',[0 0 0],'xlim', [0, timeout_used], 'ylim', [0, 120] * scaler-388.2081);
hold on;
plotHandle4 = plot(axesHandle4,i_real,Ventricle(1,1),'Marker','.','LineWidth',1,'Color',[0 1 0]);
xlabel('Time in seconds','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
ylabel('Aorta Pressure in mmHg','FontWeight','bold','FontSize',10,'Color',[1 1 0]);
title('Aorta Pressure','FontSize',15,'Color',[1 1 0]);

PropName1(1)={'YData'};
PropName1(2)={'Color'};
PropName1(3)={'XData'};
PropName2(1)={'YData'};
PropName2(2)={'Color'};
PropName2(3)={'XData'};
PropVal1(1,2)={[0 1 0]};
PropVal1(2,2)={[0 1 0]};
PropVal2(1,2)={[0 1 0]};
PropVal2(2,2)={[0 1 0]};
    
% start to collect data and plot the real-time waveform    
for i = 2:(timeout_used  * sample_rate)
    fprintf(fid, '%c\r','s');
	pause(0.001);
    f_test = fscanf(fid,'%s');    
    if ((~isempty(f_test)) && (f_test(1) == 'H'))
        i_real = i_real + 1;
        H = find(f_test == 'H');
        L = find(f_test == 'L');
        V = find(f_test == 'V');
        A = find(f_test == 'A');
        U = find(f_test == 'U');
        High_tank(i_real,1) = str2double(f_test((H + 1):(L - 1)))*scaler-388.2081;
        Low_tank(i_real,1) = str2double(f_test((L + 1):(V - 1)))*scaler-388.2081;
        Ventricle(i_real,1) = str2double(f_test((V + 1):(A - 1)))*scaler-388.2081;  
        Aorta(i_real,1) = str2double(f_test((A + 1):(U - 1)))*scaler-388.2081;         
        PumpSignal(i_real,1) = str2double(f_test((U + 1):(length(f_test) - 1)))*200;
        
        PropVal1(1,1)={High_tank(1:i_real)};
        PropVal1(1,3)={[1:i_real] / sample_rate};
        PropVal1(2,1)={PumpSignal(1:i_real)};
        PropVal1(2,3)={[1:i_real] / sample_rate};
        set(plotHandle1,PropName1,PropVal1);
        set(plotHandle2,'YData',Low_tank(1:i_real),'XData',[1:i_real] / sample_rate);
        
        PropVal2(1,1)={Low_tank(1:i_real)};
        PropVal2(1,3)={[1:i_real] / sample_rate};
        PropVal2(2,1)={PumpSignal(1:i_real)};
        PropVal2(2,3)={[1:i_real]/ sample_rate};
        set(plotHandle2,PropName2,PropVal2);
        
        set(plotHandle3,'YData',Ventricle(1:i_real),'XData',[1:i_real] / sample_rate);        
        set(plotHandle4,'YData',Aorta(1:i_real),'XData',[1:i_real] / sample_rate); 
    end    
end  
       
timelength = timeout_used / 60;
% savefile is the name of the saved file
savefile = sprintf('%d_%2d_%2d_%2d_%2d_%d', year(now),month(now),day(now),hour(now),minute(now),timelength);
cprintf(-[1,0,1],'\n\rThe format of the name of the saved file is "Year_Month_Day_Hour_Minute_Timelength"\n\r"');
% save the four vectors to a mat file
save(savefile, 'High_tank', 'Low_tank', 'Ventricle', 'Aorta');
%clear all the internal variables
clear f f1 H L A V over timelength timeout_used send_packet t_Average;
clear figureHandle1 figureHandle2 figureHandle3 figureHandle4;
clear plotHandle1 plotHandle2 plotHandle3 plotHandle4;
clear axesHandle1 axesHandle2 axesHandle3 axesHandle4;
clear i i_real check check_flag s savefile timeout f_test;
clear Square_current Square_used Square_location scaler;
clear Bias_used Gain_total P_diff_used PropName1 PropName2 PropVal1 PropVal2
clear R_Calib R_used U a_Transducer b_Transducer dP_MA dV_MA R_Use delta_used
% close the serial port
fclose(fid);
delete(fid);
clear fid
